@@ -1,5 +1,5 @@
 /*
-    htpdate v1.3.4
+    htpdate v1.3.5
 
     Eddy Vervest <eddy@vervest.org>
     http://www.vervest.org/htp
@@ -61,7 +61,7 @@
 #include <openssl/ssl.h>
 #endif
 
-#define VERSION                  "1.3.4"
+#define VERSION                  "1.3.5"
 #define MAX_HTTP_HOSTS           16                /* 16 web servers */
 #define DEFAULT_HTTP_PORT        "80"
 #define DEFAULT_PROXY_PORT       "8080"
@@ -406,10 +406,11 @@ static double getHTTPdate(
         #endif
             rc = sendHEAD(server_s, headrequest, buffer);
 
-        if (!rc)
+        if (!rc) {
             printlog(1, "error from %s:%s", host, port );
-
-        if (rc) {
+            offset = LLONG_MAX;
+            break;
+        } else {
             clock_gettime(CLOCK_REALTIME, &now);
 
             /* rtt contains round trip time in nanoseconds */
@@ -438,7 +439,8 @@ static double getHTTPdate(
                     remote_time, rtt / (long)1e6, offset);
             } else {
                 printlog(1, "%s no timestamp", host);
-                return(ERR_TIMESTAMP);
+                offset = LLONG_MAX;
+                break;
             }
         }                           /* bytes received */
         precision--;
@@ -454,6 +456,7 @@ static double getHTTPdate(
 
     /* Rounding */
     if (debug) printlog(0, "when: %ld, nap: %ld", when, nap);
+    if (offset == LLONG_MAX) return(ERR_TIMESTAMP);
     if (when + nap == 1e9 && offset == 0) return 0;
 
     /* Return the time delta between web server time (timevalue)
@@ -467,6 +470,17 @@ static double getHTTPdate(
 }
 
 
+static int setstatus(int precision) {
+    struct timex txc = {0};
+
+    txc.modes = MOD_STATUS | (1000000 >> precision) | MOD_MAXERROR;
+    txc.status &= ~STA_UNSYNC;
+    printlog(0, "Set clock synchronized");
+
+    return(adjtimex(&txc));
+}
+
+
 static int setclock(double timedelta, int setmode) {
     struct timespec now;
     struct timeval  timeofday;
@@ -544,7 +558,7 @@ static int init_frequency(char *driftfile) {
 }
 
 
-static int htpdate_adjtimex(double drift, char *driftfile) {
+static int htpdate_adjtimex(double drift, char *driftfile, float confidence) {
     struct timex    tmx;
     long            freq;
     FILE            *fp;
@@ -557,7 +571,7 @@ static int htpdate_adjtimex(double drift, char *driftfile) {
     freq = (long)(65536e6 * drift);
 
     /* Weighted average of current and new frequency */
-    tmx.freq = tmx.freq + (freq >> 3);
+    tmx.freq = tmx.freq + freq * confidence;
     if ((tmx.freq < -MAX_DRIFT) || (tmx.freq > MAX_DRIFT))
         tmx.freq = sign(tmx.freq) * MAX_DRIFT;
 
@@ -583,7 +597,7 @@ static int htpdate_adjtimex(double drift, char *driftfile) {
 
 static void showhelp() {
     puts("htpdate version "VERSION"\n\
-Usage: htpdate [-046acdhlnqstvxD] [-f driftfile] [-i pidfile] [-m minpoll]\n\
+Usage: htpdate [-046acdhlnqstvxDF] [-f driftfile] [-i pidfile] [-m minpoll]\n\
          [-M maxpoll] [-p precision] [-P <proxyserver>[:port]]\n\
          [-u user[:group]] <URL> ...\n\n\
   -0    HTTP/1.0 request\n\
@@ -795,6 +809,8 @@ int main(int argc, char *argv[]) {
             exit(0);
         case 'x':               /* adjust time and clock frequency */
             setmode = 3;
+	    if (maxsleep > 14400) maxsleep = 14400;
+	    if (precision < 7) precision = 7;
             break;
         case 'D':               /* run as daemon */
             daemonize = 1;
@@ -920,7 +936,7 @@ int main(int argc, char *argv[]) {
         /* Check if we have at least one valid response */
         if (goodtimes) {
 
-            timeavg = sumtimes / goodtimes;
+            timeavg = sumtimes / goodtimes / 2;
 
             if (debug > 1)
                 printlog(0, "#: %d, mean: %.3f, average: %.3f", goodtimes, mean, timeavg);
@@ -943,7 +959,7 @@ int main(int argc, char *argv[]) {
                         if (setmode == 3) {
                             starttime = time(NULL);
                             /* Adjust the clock frequency */
-                            if (htpdate_adjtimex(drift, driftfile) < 0)
+                            if (htpdate_adjtimex(drift, driftfile, (float)sleeptime / (float)maxsleep) < 0)
                                 printlog(1, "Frequency change failed");
 
                             /* Drop root privileges again */
@@ -962,6 +978,7 @@ int main(int argc, char *argv[]) {
             } else {
                 /* Increase polling interval */
                 if (sleeptime < maxsleep) sleeptime <<= 1;
+		if (setmode == 3) setstatus(precision);
             }
 
             if (daemonize || foreground) {
